package com.mofang.travel.marketing.base.config;

import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.HashMap;

/**
 * 请求日志
 *
 * @Author zy
 **/
@Slf4j
@Aspect
@Component
public class LogAspect {

    @Pointcut("execution(* *..*Controller.*(..))")
    public void pointCut() {
    }

    @Around("pointCut()")
    public Object around(ProceedingJoinPoint pjp) throws Throwable {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        long start = System.currentTimeMillis();
        Object obj = null;
        try {
            obj = pjp.proceed();
        } catch (Throwable throwable) {
            throw throwable;
        } finally {
            long costTime = System.currentTimeMillis() - start;

            Enumeration<String> enu = request.getParameterNames();
            HashMap<String, String> params = new HashMap<>();
            while (enu.hasMoreElements()) {
                String paraName = enu.nextElement();
                params.put(paraName, request.getParameter(paraName));
            }

            if (log.isInfoEnabled()) {
                String responseJson = JSONUtil.toJsonStr(obj);
                log.info("调用 [ " + request.getRequestURL() + " ] , request= " + JSONUtil.toJsonStr(params) + ", response= " + responseJson + ", 耗时= " + costTime + " ms");
            }

        }
        return obj;
    }

}
